// CardDlg.cpp : implementation file
//
 
#include "stdafx.h"
#include "d3dtypes.h"					// Include for RGBA_MAKE macro
#include <ISTBTDDExt.h>
#include "tulsa sample app.h"
#include "mainfrm.h"
#include "CardDlg.h"
#include "config.h"
#include "InitCardDlg.h"
#include "GraphicsOptionsDlg.h"
#include "AnalogVideoEncoderTestDlg.h"
#include "MPEGDecControlDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#undef BACK_BUFFER						// Enable a back buffer for the frame buffer
#define SHOW_FIRST_PRESET_ON_INIT		// Define to have the first preset used on init
#undef TEST_FF_ALPHA_PRESERVATION

#define MAX_CARD_INIT_WAITS		100		// Check this often (in ms) while waiting
										//  for card initialization to complete
#define CARD_INIT_RETRY_DELAY	(20*10)	// Retry up to this many times while waiting for
										//  card to finish initialization
#define STATUS_UPDATE_PERIOD	500		// Time in ms between status updates

static BOOL bInUpdateStatus = FALSE;


IMPLEMENT_DYNCREATE(CCardDlg, CPropertyPage)


void CCardDlg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCardDlg)
	//}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////
// Dialog class constructors/destructors
/////////////////////////////////////////////////////////////////////////////
CCardDlg::CCardDlg(int CardNumber) : CPropertyPage(CCardDlg::IDD)
{
	//{{AFX_DATA_INIT(CCardDlg)
	//}}AFX_DATA_INIT
	m_CardNumber = CardNumber;
	pAudioControl = NULL;
	pAnalogVideoDecoderControl = NULL;
	pAnalogVideoEncoderControl = NULL;
	pBoardControl = NULL;
	pGenlockControl = NULL;
	pMPEGDecoderControl = NULL;
	pVBIControl = NULL;
	pVideoControl = NULL;
	pDebug = NULL;

	for (int i = 0; i < NUM_SUB_COMMANDS; i++) {
		m_hStream[i] = 0;		
	}

	lpDirectDrawRGB = NULL;
	lpDirectDrawYUV = NULL;

	lpDD2RGB = NULL;
	lpDD2YUV = NULL;

	lpDDSPrimaryRGB = NULL;
	lpDDSPrimaryYUV = NULL;
 
	lpDDSOverlayVScroll = NULL;
	lpDDSOverlayHScroll = NULL;
	lpDDSTitle = NULL;
	lpDDSTest = NULL;
	lpDDSOverlayTimebar = NULL;
	for (i = 0; i < MAX_VIDEO_PORTS; i++) {
		lpDDVideoPortYUV[i] = NULL;
		bVideoTestStartedYUV[i] = FALSE;
	}

	lpDDSOverlayVideo = NULL;
	lpDDVideoPortRGB = NULL;
	bVideoTestStartedRGB = FALSE;

	RGBVideoZOrderFlags = 0;
	VScrollZOrderFlags = 0;
	HScrollZOrderFlags = 0;
	TitleOvlyZOrderFlags = 0;

	HorzDisplayCount = 0;
	bEnableHorzScrollThread = FALSE;
	bHorzScrollThreadFinished = FALSE;
	hEventHorzScroll = NULL;
	hEventHorzFF = NULL;
	HScrollReady = FALSE;

	VertDisplayCount = 0; 
	bEnableVertScrollThread = FALSE;
	bVertScrollThreadFinished = FALSE;
	hEventVertScroll = NULL;
	hEventVertFF = NULL;
	VScrollReady = FALSE;

	m_bVSUpdate = FALSE;
	m_bHSUpdate = FALSE;

	pYUVPriVideoOvly = NULL;
	pYUVSecVideoOvly = NULL;
	pRGBVideoOvly = NULL;

	pVScrollOvly = NULL;
	pHScrollOvly = NULL;
	pTitleOvly = NULL;

	AlphaOptions.AlphaValue = 0x0;
	AlphaOptions.bHorzGradient = FALSE;
	AlphaOptions.bVertGradient = FALSE;
	AlphaOptions.bInvertAlpha = FALSE;
	AlphaOptions.bTransitionAlpha = FALSE;
	AlphaOptions.TransitionVerticals = 2;
	m_bLastAlpha = 0;

	m_VScrollDibInfo.hDib = NULL;

	m_VideoMode = VIDEO_MODE_NTSC;
	m_LastVideoMode = VIDEO_MODE_NTSC;

	UsePreset(0);
}

CCardDlg::~CCardDlg()
{
	DeletePreset();
}


/////////////////////////////////////////////////////////////////////////////
// Initialize the Card control dialog box
/////////////////////////////////////////////////////////////////////////////
BOOL CCardDlg::OnInitDialog() 
{
	TULSA_RESULT result;
	CPropertyPage::OnInitDialog();

	CWinApp* pApp = AfxGetApp();

	result = pBoardControl->QueryInterface(IID_ISTBTAudioControl, (LPVOID *)&pAudioControl);
	result = pBoardControl->QueryInterface(IID_ISTBTAnalogVideoDecoderControl, (LPVOID *)&pAnalogVideoDecoderControl);
	result = pBoardControl->QueryInterface(IID_ISTBTAnalogVideoEncoderControl, (LPVOID *)&pAnalogVideoEncoderControl);
	result = pBoardControl->QueryInterface(IID_ISTBTGenlockControl, (LPVOID *)&pGenlockControl);
	result = pBoardControl->QueryInterface(IID_ISTBTMPEGDecoderControl, (LPVOID *)&pMPEGDecoderControl);
	result = pBoardControl->QueryInterface(IID_ISTBTVBIControl, (LPVOID *)&pVBIControl);
	result = pBoardControl->QueryInterface(IID_ISTBTVideoControl, (LPVOID *)&pVideoControl);
	result = pBoardControl->QueryInterface(IID_ISTBTDebug, (LPVOID *)&pDebug);
	result = pBoardControl->QueryInterface(IID_ISTBTTest, (LPVOID *)&pTest);

	pMpegStatus = new CMpegStatus(NULL);
	pVerticalStatus = new CVerticalStatus(this);

	pDebug->DebugOpenLog(0);		// Open logging facility of Control DLL

									// Init the configuration structure
	char szBuffer[60];
	m_Config.bDisableAVSync = pApp->GetProfileInt("Settings", PrependCardID(szBuffer, "DisableAVSync"), 0);
	m_Config.bDisableGenLock = pApp->GetProfileInt("Settings", PrependCardID(szBuffer, "DisableGenLock"), 0);

	m_Config.DiskReadConfig.BufferCount = pApp->GetProfileInt("Settings", PrependCardID(szBuffer, "DiskBufferCount"), 4);		
	m_Config.DiskReadConfig.BufferSize = pApp->GetProfileInt("Settings", PrependCardID(szBuffer, "DiskBufferSize"), 0x80000);		

	m_VideoMode = m_Config.VideoMode = (VIDEO_MODE)pApp->GetProfileInt("Settings", PrependCardID(szBuffer, "DefVideoMode"), (DWORD)VIDEO_MODE_NTSC);

	m_Config.AudioDelay = 33;
	m_Config.AudioSource = AUDIO_ANALOG_IN;
	m_Config.AudioVolume[AUDIO_MPEG_DEC] = 63;
	m_Config.AudioVolume[AUDIO_ANALOG_IN] = 63;
	m_Config.AudioVolume[AUDIO_EXT_DIGITAL_IN] = 63;
	m_Config.AudioPhase[AUDIO_MPEG_DEC] = 0;
	m_Config.AudioPhase[AUDIO_ANALOG_IN] = 0;
	m_Config.AudioPhase[AUDIO_EXT_DIGITAL_IN] = 0;
	m_Config.bLine21PassThru = TRUE;
	m_Config.VideoSource[VIDEO_PORT_YUV_PRI] = VIDEO_ANALOG_IN;
	m_Config.VideoSource[VIDEO_PORT_YUV_SEC] = VIDEO_ANALOG_IN;
	m_Config.VideoSource[VIDEO_PORT_RGB] = VIDEO_ANALOG_IN;
	m_Config.GenlockSource = GENLOCK_SRC_VIDEO_IN;
	m_Config.bACCEnable = m_bACCEnable = FALSE;
	m_Config.ThreadPriority = THREAD_PRIORITY_ABOVE_NORMAL;

	m_bBypass = FALSE;				// Currently board defaults to non-bypassed after init
									// Show message box while initializing card
	CInitCardDlg dlg(m_CardNumber);
									// Initialize the card
	TULSA_RESULT InitResult = InitCard();

	if (!pBoardControl)				// Make sure init was successful
		return TRUE;

	SetAnalogAudioVolume();			// Initialize dialog controls
	SetMPEGAudioVolume();
	SetAudioSwitch();
	SetVideoSwitch();
	SetAudioDelay();
	SetBypassSwitch();
	SetACCMode();
	
	m_PictureAudSwitch[0].CreateFromBitmap(IDB_AUD_SWITCH_MPEG);
	m_PictureAudSwitch[1].CreateFromBitmap(IDB_AUD_SWITCH_ANALOG);

	m_PictureVidSwitch[0].CreateFromBitmap(IDB_VID_SWITCH_0);
	m_PictureVidSwitch[1].CreateFromBitmap(IDB_VID_SWITCH_1);
	m_PictureVidSwitch[2].CreateFromBitmap(IDB_VID_SWITCH_2);
	m_PictureVidSwitch[3].CreateFromBitmap(IDB_VID_SWITCH_3);
	m_PictureVidSwitch[4].CreateFromBitmap(IDB_VID_SWITCH_4);
	m_PictureVidSwitch[5].CreateFromBitmap(IDB_VID_SWITCH_5);
	m_PictureVidSwitch[6].CreateFromBitmap(IDB_VID_SWITCH_6);
	m_PictureVidSwitch[7].CreateFromBitmap(IDB_VID_SWITCH_7);

	m_PictureAudBypass[0].CreateFromBitmap(IDB_AUD_BYPASS_ON);
	m_PictureAudBypass[1].CreateFromBitmap(IDB_AUD_BYPASS_OFF);

	m_PictureVidBypass[0].CreateFromBitmap(IDB_VID_BYPASS_ON);
	m_PictureVidBypass[1].CreateFromBitmap(IDB_VID_BYPASS_OFF);
									// Create a timer for update notifications
	m_TimerID = SetTimer(0, STATUS_UPDATE_PERIOD, NULL);

	if (InitResult == SUCCESS) {
		SetupRGBProc();
		SetupYUVProc();
		m_LastVideoMode = m_VideoMode;
	}
									// Allow board output to be used
	pBoardControl->EnableBypass(FALSE);

	GetDlgItem(IDC_RESET)->ShowWindow(SW_HIDE);

	return TRUE;  
}


/////////////////////////////////////////////////////////////////////////////
// Cleanup on closing of dialog box
/////////////////////////////////////////////////////////////////////////////
void CCardDlg::OnDestroy() 
{
	StopPlayback();
	
	StopVertScrollTest();
	StopHorzScrollTest();
	DeleteTitleOverlay();
	StopVideoTestAllYUV();
	StopVideoTestRGB();

	if (lpDDSPrimaryRGB) {
		lpDDSPrimaryRGB->Release();
		lpDDSPrimaryRGB = NULL;
	}

	if (lpDDSPrimaryYUV) {
		lpDDSPrimaryYUV->Release();
		lpDDSPrimaryYUV = NULL;
	}

	if (lpDD2RGB) {
		lpDD2RGB->Release();
		lpDD2RGB = NULL;
	}

	if (lpDD2YUV) {
		lpDD2YUV->Release();
		lpDD2YUV = NULL;
	}

	if (lpDirectDrawRGB) {
		lpDirectDrawRGB->Release();
		lpDirectDrawRGB = NULL;
	}

	if (lpDirectDrawYUV) {
		lpDirectDrawYUV->Release();
		lpDirectDrawYUV = NULL;
	}

	if (m_TimerID) {
		KillTimer(m_TimerID);
	}


	CPropertyPage::OnDestroy();

	if (pBoardControl)
	{
		pBoardControl->Release();
		pAudioControl->Release();
		pAnalogVideoDecoderControl->Release();
		pAnalogVideoEncoderControl->Release();
		pGenlockControl->Release();
		pMPEGDecoderControl->Release();
		pVBIControl->Release();
		pVideoControl->Release();
		pTest->Release();
		pDebug->Release();
		pBoardControl = NULL;
	}
	delete pMpegStatus;
	delete pVerticalStatus;
}


/////////////////////////////////////////////////////////////////////////////
// Handle timer notification for dialog updates
/////////////////////////////////////////////////////////////////////////////
void CCardDlg::OnTimer(UINT nIDEvent) 
{
	UpdateStatus(0, 0);
	CPropertyPage::OnTimer(nIDEvent);
}


/////////////////////////////////////////////////////////////////////////////
// Use the Control Object to Initialize the board
/////////////////////////////////////////////////////////////////////////////
TULSA_RESULT CCardDlg::InitCard(void)
{
	TULSA_RESULT result;
	char buffer[60];

	BeginWaitCursor();

	result = pBoardControl->InitBoard(&m_Config);
									// Initialize the card
	if (result != SUCCESS) {
		sprintf(buffer, "Card %d initialization failed.  Error code: %08.8x", m_CardNumber, result);
		AfxMessageBox(buffer, MB_OK | MB_ICONEXCLAMATION);
	} else {
		TULSA_BOARD_STATUS Status;
		TULSA_BOARD_VERSION Version;
									// Wait for initialization to complete
		for (int i = 0; i < MAX_CARD_INIT_WAITS; i++) {
			pBoardControl->GetBoardStatus(&Status);
			pBoardControl->GetBoardVersion(&Version);
			if (Status.State == BOARD_STAT_RUNNING) {
				break;
			}
			Sleep(CARD_INIT_RETRY_DELAY);
		}

		if (i == MAX_CARD_INIT_WAITS) {
			AfxMessageBox("Error!  Card did not initialize correctly.", MB_OK | MB_ICONEXCLAMATION);
		}

	}
	EndWaitCursor();
	return(SUCCESS);
}


/////////////////////////////////////////////////////////////////////////////
// Init the RGB graphics processor
/////////////////////////////////////////////////////////////////////////////
void CCardDlg::SetupRGBProc()
{
	DDSURFACEDESC   ddsd;
	HRESULT result;

	BackGroundColor = RGBA_MAKE(80, 0, 0, 0);

    result = lpDirectDrawRGB->QueryInterface(IID_IDirectDraw2, (void **)&lpDD2RGB);
	if(!FAILED(result)) {
		if (m_VideoMode == VIDEO_MODE_NTSC) {
			lpDD2RGB->SetDisplayMode(720, 480, 32, 60, 0);
		} else {
			lpDD2RGB->SetDisplayMode(720, 576, 32, 50, 0);
		}
	}

	INIT_DIRECTDRAW_STRUCT(ddsd);
	ddsd.dwFlags = DDSD_CAPS;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

#ifdef BACK_BUFFER
	ddsd.dwFlags |= DDSD_BACKBUFFERCOUNT;
	ddsd.ddsCaps.dwCaps |= DDSCAPS_FLIP | DDSCAPS_COMPLEX;
	ddsd.dwBackBufferCount = 1;
#endif

	result = lpDD2RGB->CreateSurface(&ddsd, &lpDDSPrimaryRGB, NULL);
	if(!FAILED(result)) {
		FillSurface(lpDDSPrimaryRGB, BackGroundColor);		

		DDSURFACEDESC SurfaceDesc;
		if (lpDDSPrimaryRGB->Lock(NULL, &SurfaceDesc, 0, NULL) == DD_OK) {
										// Copy bitmap to surface
			CopyBitmapFromFile(&SurfaceDesc, &PriSurfaceContents, TRUE);
		}
		lpDDSPrimaryRGB->Unlock(NULL);
	}


#ifdef SHOW_FIRST_PRESET_ON_INIT
	if (pTitleOvly->m_bShow) {
		ShowTitleOverlay();
	}

	if (pHScrollOvly->m_bShow) {
		StartHorzScrollTest();
	}

	if (pVScrollOvly->m_bShow) {
		StartVertScrollTest();
	}

	if (pRGBVideoOvly->m_bShow) {
		StartVideoTestRGB();
	}

	ApplyAlphaSettings(FALSE);
#else

	LPTULSASURFACEEXT lpTulsaSurfaceExt;

	result = CoCreateInstance(CLSID_STBTDDExtension, NULL, 
		CLSCTX_INPROC, IID_ISTBTDDSurfaceExt, (LPVOID *)&lpTulsaSurfaceExt);
	if (result != DD_OK) {
		return;
	}
										// Assign surface to extend
	lpTulsaSurfaceExt->AssignSurface(lpDDSPrimaryRGB);
										// Specify the handling for the alpha channel
	SURFACEDESCEXT SurfaceDescExt;
	INIT_DIRECTDRAW_STRUCT(SurfaceDescExt);
	SurfaceDescExt.dwFlags = 0;
	SurfaceDescExt.bPreserveDestAlpha = FALSE;	//N/A for primary surface
	SurfaceDescExt.bInvertAlpha = TRUE;
	lpTulsaSurfaceExt->SetSurfaceDescExt(&SurfaceDescExt);

	lpTulsaSurfaceExt->Release();

#endif
}


/////////////////////////////////////////////////////////////////////////////
// Init the YUV graphics processor
/////////////////////////////////////////////////////////////////////////////
void CCardDlg::SetupYUVProc()
{
	DDSURFACEDESC   ddsd;
	HRESULT result;

	BackGroundColor = RGBA_MAKE(80, 0, 0, 0);

    result = lpDirectDrawYUV->QueryInterface(IID_IDirectDraw2, (void **)&lpDD2YUV);
	if(!FAILED(result)) {
		if (m_VideoMode == VIDEO_MODE_NTSC) {
			lpDD2YUV->SetDisplayMode(720, 480, 16, 60, 0);
		} else {
			lpDD2YUV->SetDisplayMode(720, 576, 16, 50, 0);
		}
	}

	INIT_DIRECTDRAW_STRUCT(ddsd);
	ddsd.dwFlags = DDSD_CAPS;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

#ifdef BACK_BUFFER
	ddsd.dwFlags |= DDSD_BACKBUFFERCOUNT;
	ddsd.ddsCaps.dwCaps |= DDSCAPS_FLIP | DDSCAPS_COMPLEX;
	ddsd.dwBackBufferCount = 1;
#endif

	result = lpDD2YUV->CreateSurface(&ddsd, &lpDDSPrimaryYUV, NULL);
	if(!FAILED(result)) {
		DrawYUVColorBars(lpDDSPrimaryYUV);		
	}

	if (pYUVPriVideoOvly->m_bShow) {
		StartVideoTestYUV(PRIMARY_VIDEO_PORT);
	}

	if (pYUVSecVideoOvly->m_bShow) {
		StartVideoTestYUV(SECONDARY_VIDEO_PORT);
	}

}

/////////////////////////////////////////////////////////////////////////////
// Apply the curent graphics settings (as changed by the Graphics Options Dialog box)
/////////////////////////////////////////////////////////////////////////////
void CCardDlg::ApplyGraphicsSettings()
{
	HRESULT result;
	DDSURFACEDESC SurfaceDesc;
	LPTULSASURFACEEXT lpTulsaSurfaceExt;
	LPDIRECTDRAWSURFACE lpDDSBack;
	DDSCAPS ddscaps;
	ddscaps.dwCaps = DDSCAPS_BACKBUFFER;

	if (m_LastVideoMode != m_VideoMode) {
		StopVertScrollTest();
		StopHorzScrollTest();
		DeleteTitleOverlay();
		StopVideoTestAllYUV();
		StopVideoTestRGB();

		if (lpDD2RGB) {
			lpDD2RGB->Release();
		}

		if (lpDD2YUV) {
			lpDD2YUV->Release();
		}
		SetupRGBProc();
		SetupYUVProc();
		m_LastVideoMode = m_VideoMode;
	}

		// Update the RGB processor...
	result = CoCreateInstance(CLSID_STBTDDExtension, NULL, 
		CLSCTX_INPROC, IID_ISTBTDDSurfaceExt, (LPVOID *)&lpTulsaSurfaceExt);
	if (result != DD_OK) {
		return;
	}

#ifdef BACK_BUFFER
	ApplyAlphaSettings(TRUE);
#else
	ApplyAlphaSettings(FALSE);
#endif

	if (bShowBackGnd) {
		lpDDSBack = lpDDSPrimaryRGB;
#ifdef BACK_BUFFER
		if (lpDDSPrimaryRGB->GetAttachedSurface(&ddscaps, &lpDDSBack) != DD_OK) {
			return;
		}
#endif
		if (lpDDSBack->Lock(NULL, &SurfaceDesc, 0, NULL) == DD_OK) {
									// Copy bitmap to surface
			CopyBitmapFromFile(&SurfaceDesc, &PriSurfaceContents, bBkGndUpdateAlpha);
		}
		lpDDSBack->Unlock(NULL);

		if (bBackGndFlickerFilter) {
											// Assign surface being extended
			lpTulsaSurfaceExt->AssignSurface(lpDDSBack);

			POINT FilterStart;
			SIZE FilterSize;
			FilterStart.x = 0;
			FilterStart.y = 0;
			FilterSize.cx = SurfaceDesc.dwWidth;
			FilterSize.cy = SurfaceDesc.dwHeight - 1;	// Don't filter the last line since it depends upon
													//  the next line after it
#ifdef TEST_FF_ALPHA_PRESERVATION
			if (lpTulsaSurfaceExt->StartFlickerFilterEx(&FilterStart, &FilterSize, NULL, TRUE) == DD_OK) {
#else
			if (lpTulsaSurfaceExt->StartFlickerFilter(&FilterStart, &FilterSize, NULL) == DD_OK) {
#endif
				BOOL bFinished = FALSE;
				DWORD Line;
				do {
					lpTulsaSurfaceExt->GetFlickerFilterStatus(&bFinished, &Line);
					Sleep(10);
				} while (!bFinished);
			}
		}
	}
											// Assign surface being extended
	lpTulsaSurfaceExt->AssignSurface(lpDDSPrimaryRGB);

	lpTulsaSurfaceExt->SuspendUpdates(TRUE);

	if (!pRGBVideoOvly->m_bShow) {
		StopVideoTestRGB();
	}

	if (!pVScrollOvly->m_bShow) {
		StopVertScrollTest();
	}

	if (!pHScrollOvly->m_bShow) {
		StopHorzScrollTest();
	}

	if (!pTitleOvly->m_bShow) {
		DeleteTitleOverlay();
	}

	if (pRGBVideoOvly->m_bShow) {
		if (bVideoTestStartedRGB) {
			UpdateVideoTestRGB();
		} else {
			StartVideoTestRGB();
		}
	}

	if (pTitleOvly->m_bShow) {
		if (lpDDSTitle) {
			UpdateTitleOverlay();
		} else {
			ShowTitleOverlay();
		}
	}

	if (pHScrollOvly->m_bShow) {
		if (HorzScrollEnabled()) {
			UpdateHorzScrollTest();
		} else {
			StartHorzScrollTest();
		}
	}


	if (pVScrollOvly->m_bShow) {
		if (VertScrollEnabled()) {
			UpdateVertScrollTest();
		} else {
			StartVertScrollTest();
		}
	}

#ifdef BACK_BUFFER
	lpDDSPrimaryRGB->Flip(NULL, 0);
#endif
	lpTulsaSurfaceExt->SuspendUpdates(FALSE);
	lpTulsaSurfaceExt->Release();

		// Update the YUV processor...
	if (!pYUVPriVideoOvly->m_bShow) {
		StopVideoTestYUV(PRIMARY_VIDEO_PORT);
	}

	if (!pYUVSecVideoOvly->m_bShow) {
		StopVideoTestYUV(SECONDARY_VIDEO_PORT);
	}

	if (pYUVPriVideoOvly->m_bShow) {
		if (bVideoTestStartedYUV[PRIMARY_VIDEO_PORT]) {
			UpdateVideoTestYUV(PRIMARY_VIDEO_PORT);
		} else {
			StartVideoTestYUV(PRIMARY_VIDEO_PORT);
		}
	}

	if (pYUVSecVideoOvly->m_bShow) {
		if (bVideoTestStartedYUV[SECONDARY_VIDEO_PORT]) {
			UpdateVideoTestYUV(SECONDARY_VIDEO_PORT);
		} else {
			StartVideoTestYUV(SECONDARY_VIDEO_PORT);
		}
	}

}


/////////////////////////////////////////////////////////////////////////////
// Disable the curent graphics settings 
/////////////////////////////////////////////////////////////////////////////
void CCardDlg::DisableGraphicsSettings()
{
	StopVertScrollTest();
	StopHorzScrollTest();
	DeleteTitleOverlay();
	StopVideoTestAllYUV();
	StopVideoTestRGB();
}



/////////////////////////////////////////////////////////////////////////////
// Update the dialog box
/////////////////////////////////////////////////////////////////////////////
LRESULT CCardDlg::UpdateStatus(WPARAM wParam, LPARAM lParam)
{
	DECODE_STATUS DecodeStatus;
	char szBuffer[150];
	char *pstr;

	if (m_hWnd == NULL) {	
		return 0;								// Not visible yet
	}

	if (bInUpdateStatus) {
		return 0;
	}
	bInUpdateStatus = TRUE;

	pMPEGDecoderControl->GetStatus(0, &DecodeStatus);
	sprintf(szBuffer, "%d", DecodeStatus.CurrentVertical);
	SetDlgItemText(IDC_CURR_VERTICAL_ID, szBuffer); 

	TULSA_BOARD_STATUS CardStatus;
	pBoardControl->GetBoardStatus(&CardStatus);

	switch (CardStatus.State) {
	case BOARD_STAT_RESET:
		pstr = "Reset";
		break;
	case BOARD_STAT_MONITOR:
		pstr = "Monitor";
		break;
	case BOARD_STAT_RUNNING:
		pstr = "Running";
		break;
	case BOARD_STAT_UNKNOWN:
	default:
		pstr = "Unknown";
		break;
	}
	SetDlgItemText(IDC_BOARD_STATUS, pstr);


	LPTULSASURFACEEXT lpTulsaSurfaceExt;
	DWORD RGBProcUsage = 0;
	DWORD YUVProcUsage = 0;

	HRESULT result = CoCreateInstance(CLSID_STBTDDExtension, NULL, 
		CLSCTX_INPROC, IID_ISTBTDDSurfaceExt, (LPVOID *)&lpTulsaSurfaceExt);
	if (result == DD_OK) {		
		if (lpDDSPrimaryRGB) {
			lpTulsaSurfaceExt->AssignSurface(lpDDSPrimaryRGB);
			lpTulsaSurfaceExt->GetGraphicsLoading(&RGBProcUsage);
		}
		if (lpDDSPrimaryYUV) {
			lpTulsaSurfaceExt->AssignSurface(lpDDSPrimaryYUV);
			lpTulsaSurfaceExt->GetGraphicsLoading(&YUVProcUsage);
		}
		lpTulsaSurfaceExt->Release();
	}
	sprintf(szBuffer, "%d%%", RGBProcUsage);
	SetDlgItemText(IDC_GRAPHICS_LOADING_RGB, szBuffer); 

	sprintf(szBuffer, "%d%%", YUVProcUsage);
	SetDlgItemText(IDC_GRAPHICS_LOADING_YUV, szBuffer); 

	bInUpdateStatus = FALSE;
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// Open the Configuration Dialog box
/////////////////////////////////////////////////////////////////////////////
void CCardDlg::OnConfig() 
{
	CConfig dlg;
	dlg.m_AVSync = !m_Config.bDisableAVSync;
	dlg.m_GenLock = !m_Config.bDisableGenLock;
	dlg.m_VideoMode = m_Config.VideoMode;
	dlg.m_DiskBufferCount = m_Config.DiskReadConfig.BufferCount;
	dlg.m_DiskBufferSize = m_Config.DiskReadConfig.BufferSize;

	dlg.m_MemorySize =0x0800000;
	dlg.m_GenLockSupport = TRUE;

	if (dlg.DoModal() == IDOK) {
		m_Config.bDisableAVSync = !dlg.m_AVSync;
		m_Config.bDisableGenLock = !dlg.m_GenLock;
		m_Config.VideoMode = (VIDEO_MODE)dlg.m_VideoMode;
		m_Config.DiskReadConfig.BufferCount = dlg.m_DiskBufferCount;
		m_Config.DiskReadConfig.BufferSize = dlg.m_DiskBufferSize;

		char szBuffer[1000];
		CWinApp* pApp = AfxGetApp();
		pApp->WriteProfileInt("Settings", PrependCardID(szBuffer, "DisableAVSync"), m_Config.bDisableAVSync);
		pApp->WriteProfileInt("Settings", PrependCardID(szBuffer, "DisableGenLock"), m_Config.bDisableGenLock);
		pApp->WriteProfileInt("Settings", PrependCardID(szBuffer, "DefVideoMode"), m_Config.VideoMode);

		pApp->WriteProfileInt("Settings", PrependCardID(szBuffer, "DiskBufferCount"),	m_Config.DiskReadConfig.BufferCount);
		pApp->WriteProfileInt("Settings", PrependCardID(szBuffer, "DiskBufferSize"), m_Config.DiskReadConfig.BufferSize);

		sprintf(szBuffer, "Would you like to reset card %d so that these changes will be used?", m_CardNumber);
		if (MessageBox(szBuffer, "Tulsa Sample Application", MB_ICONQUESTION | MB_YESNO) == IDYES) {
			OnReset();
		}
	}
}


/////////////////////////////////////////////////////////////////////////////
// Open the Graphics Options Dialog box
/////////////////////////////////////////////////////////////////////////////
void CCardDlg::OnGraphicsOptions() 
{
	CGraphicsOptionsDlg dlg;

	dlg.m_pCardDlg = this;

	if (dlg.DoModal() == IDOK) {
		ApplyGraphicsSettings();
	}
}


/////////////////////////////////////////////////////////////////////////////
// Open the MPEG Control Dialog box
/////////////////////////////////////////////////////////////////////////////
void CCardDlg::OnMpegDecControl() 
{
	CMPEGDecControlDlg dlg;
	for (int i = 0; i < NUM_SUB_COMMANDS; i++) {
		dlg.m_hStream[i] = m_hStream[i];
	}
	dlg.m_pMpegStatus = pMpegStatus;
	dlg.pMPEGDecoderControl = pMPEGDecoderControl;
	dlg.m_CardNumber = 	m_CardNumber;

	dlg.DoModal();

	for (i = 0; i < NUM_SUB_COMMANDS; i++) {
		m_hStream[i] = dlg.m_hStream[i];
	}
}


/////////////////////////////////////////////////////////////////////////////
// Open the Video Encoder Options Dialog box
/////////////////////////////////////////////////////////////////////////////
void CCardDlg::OnVideoEncoderOptions() 
{
	CAnalogVideoEncoderTestDlg dlg;
	dlg.m_pTest = pTest;
	dlg.DoModal();	
}


/////////////////////////////////////////////////////////////////////////////
// Callbacks
/////////////////////////////////////////////////////////////////////////////
STDMETHODIMP CMpegStatus::NotifyAtStatus(DWORD hStream, DWORD FrameNumber)
{
	if (m_pParent) {
		m_pParent->NotifyAtStatus(hStream, FrameNumber);
	}
	return NOERROR;
}

STDMETHODIMP CMpegStatus::PlayStatus(DWORD hStream,	PDECODE_STATUS pStatus)
{
	if (m_pParent) {
		m_pParent->PostMessage(WM_UPDATE_CARD_STATUS, 0, 0);
	}
	return NOERROR;
}

STDMETHODIMP CVerticalStatus::VerticalOccurred(DWORD VerticalNumber)
{
//	m_pParent->VerticalOccurred(VerticalNumber);
	return NOERROR;
}



/////////////////////////////////////////////////////////////////////////////
// Misc routines...
/////////////////////////////////////////////////////////////////////////////

void CCardDlg::ResetCard() {
	OnReset();
}

void CCardDlg::StopPlayback() 
{
	for (int i = 0; i < NUM_SUB_COMMANDS; i++) {
		if (m_hStream[i] != 0) {
			TULSA_RESULT result = pMPEGDecoderControl->Stop(m_hStream[i]);
		}
	}
}

void CCardDlg::OnReset() 
{
	StopPlayback();
	CInitCardDlg dlg(m_CardNumber);
	InitCard();	
	OnBypass();
}

void CCardDlg::SaveSettings() 
{
}

void CCardDlg::DeletePreset()
{
	if (pYUVPriVideoOvly) {
		delete pYUVPriVideoOvly;
		pYUVPriVideoOvly = NULL;
	}
	if (pYUVSecVideoOvly) {
		delete pYUVSecVideoOvly;
		pYUVSecVideoOvly = NULL;
	}
	if (pRGBVideoOvly) {
		delete pRGBVideoOvly;
		pRGBVideoOvly = NULL;
	}

	if (pVScrollOvly) {
		delete pVScrollOvly;
		pVScrollOvly = NULL;
	}
	if (pHScrollOvly) {
		delete pHScrollOvly;
		pHScrollOvly = NULL;
	}
	if (pTitleOvly) {
		delete pTitleOvly;
		pTitleOvly = NULL;
	}

	if (lpDDSPrimaryRGB) {
		FillSurface(lpDDSPrimaryRGB, 0x0);
	}

}

void CCardDlg::ReadDefaultPreset(int PresetNum)
{
	PriSurfaceContents = CGraphicsContent(SRC_BMP_FILE, ALPHA_SRC_GENERATE, "colorbars.bmp", 0, ALPHA_EFFECT_NONE);

	switch(PresetNum) {
	default:
	case 0:		

		strcpy(m_szPresetDesc, "Channel Guide");

		bShowBackGnd = FALSE;
		bBackGndFlickerFilter = FALSE;
		bBkGndUpdateAlpha = FALSE;

		AlphaOptions.AlphaValue = 0xc0;
		AlphaOptions.bHorzGradient = FALSE;
		AlphaOptions.bVertGradient = FALSE;
		AlphaOptions.bInvertAlpha = FALSE;
		AlphaOptions.bTransitionAlpha = FALSE;
		AlphaOptions.TransitionVerticals = 30;

		DeletePreset();
		pYUVPriVideoOvly =	new CVideoOverlayState	(TRUE,	CSize(720, 310),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0,		ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pYUVSecVideoOvly =	new CVideoOverlayState	(FALSE, CSize(720, 480),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0,		ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pRGBVideoOvly =		new CVideoOverlayState	(FALSE, CSize(190, 125),		CPoint(425, 60),	FALSE,	FALSE,	FALSE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0,		ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pVScrollOvly =		new CScrollOverlayState	(TRUE,	CSize(720, 240),		CPoint(0, 240),		TRUE,	FALSE,	TRUE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_BMP_FILE, ALPHA_SRC_GENERATE, "prevue.bmp",		0xd0,	ALPHA_EFFECT_NONE), 1, 2);
		pHScrollOvly =		new CScrollOverlayState	(TRUE,	CSize(720, 240),		CPoint(0, 310),		FALSE,	FALSE,	TRUE,	FALSE,	0,			TRUE,	CGraphicsContent(SRC_BMP_FILE, ALPHA_SRC_GENERATE, "tvguidlogo.bmp",	0,		ALPHA_EFFECT_NONE), 1, 1);
		pTitleOvly =		new COverlayState		(TRUE,	CSize(720, 360),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			TRUE,	CGraphicsContent(SRC_BMP_FILE, ALPHA_SRC_GENERATE, "tvguidlogo3.bmp",	0x80,	ALPHA_EFFECT_RTOL));
		break;
	case 1:		//PAL

		strcpy(m_szPresetDesc, "Config 2");

		bShowBackGnd = FALSE;
		bBackGndFlickerFilter = FALSE;
		bBkGndUpdateAlpha = FALSE;

		AlphaOptions.AlphaValue = 0xc0;
		AlphaOptions.bHorzGradient = FALSE;
		AlphaOptions.bVertGradient = FALSE;
		AlphaOptions.bInvertAlpha = FALSE;
		AlphaOptions.bTransitionAlpha = FALSE;
		AlphaOptions.TransitionVerticals = 30;

		DeletePreset();
		pYUVPriVideoOvly =	new CVideoOverlayState	(TRUE,	CSize(720, 310),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0,		ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pYUVSecVideoOvly =	new CVideoOverlayState	(FALSE, CSize(720, 480),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0,		ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pRGBVideoOvly =		new CVideoOverlayState	(FALSE, CSize(720/2, 480/2),	CPoint(360, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0,		ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pVScrollOvly =		new CScrollOverlayState	(TRUE,	CSize(720, 240),		CPoint(0, 240),		TRUE,	FALSE,	TRUE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_BMP_FILE, ALPHA_SRC_GENERATE, "prevue.bmp",		0xd0,	ALPHA_EFFECT_NONE), 1, 2);
		pHScrollOvly =		new CScrollOverlayState	(TRUE,	CSize(720, 240),		CPoint(0, 240),		FALSE,	FALSE,	TRUE,	FALSE,	0,			TRUE,	CGraphicsContent(SRC_BMP_FILE, ALPHA_SRC_GENERATE, "tvguidlogo.bmp",	0,		ALPHA_EFFECT_NONE), 1, 1);
		pTitleOvly =		new COverlayState		(TRUE,	CSize(720, 360),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			TRUE,	CGraphicsContent(SRC_BMP_FILE, ALPHA_SRC_GENERATE, "tvguidlogo3.bmp",	0x80,	ALPHA_EFFECT_RTOL));
		break;

	case 2:	//PAL 	

		strcpy(m_szPresetDesc, "Config 3");

		bShowBackGnd = FALSE;
		bBackGndFlickerFilter = FALSE;
		bBkGndUpdateAlpha = FALSE;

		AlphaOptions.AlphaValue = 0xff;
		AlphaOptions.bHorzGradient = FALSE;
		AlphaOptions.bVertGradient = FALSE;
		AlphaOptions.bInvertAlpha = FALSE;
		AlphaOptions.bTransitionAlpha = FALSE;
		AlphaOptions.TransitionVerticals = 30;

		DeletePreset();
		pYUVPriVideoOvly =	new CVideoOverlayState	(FALSE,	CSize(720, 480),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pYUVSecVideoOvly =	new CVideoOverlayState	(FALSE,	CSize(720, 480),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pRGBVideoOvly =		new CVideoOverlayState	(FALSE, CSize(720, 480),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pVScrollOvly =		new CScrollOverlayState	(FALSE, CSize(720, 160),		CPoint(0, 250),		FALSE,	FALSE,	TRUE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), 1, 2);
		pHScrollOvly =		new CScrollOverlayState	(FALSE, CSize(720, 60),			CPoint(0, 410),		FALSE,	FALSE,	TRUE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), 2, 1);
		pTitleOvly =		new COverlayState		(FALSE, CSize(313, 53),			CPoint(200, 240),	FALSE,	FALSE,	FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE));
		break;

	case 3:		

		strcpy(m_szPresetDesc, "Config 4");

		bShowBackGnd = FALSE;
		bBackGndFlickerFilter = FALSE;
		bBkGndUpdateAlpha = FALSE;

		AlphaOptions.AlphaValue = 0xc0;
		AlphaOptions.bHorzGradient = FALSE;
		AlphaOptions.bVertGradient = FALSE;
		AlphaOptions.bInvertAlpha = FALSE;
		AlphaOptions.bTransitionAlpha = FALSE;
		AlphaOptions.TransitionVerticals = 30;

		DeletePreset();
		pYUVPriVideoOvly =	new CVideoOverlayState	(FALSE,	CSize(720, 480),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0,		ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pYUVSecVideoOvly =	new CVideoOverlayState	(FALSE, CSize(720, 480),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0,		ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pRGBVideoOvly =		new CVideoOverlayState	(FALSE, CSize(720/2, 480/2),	CPoint(360, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0,		ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pVScrollOvly =		new CScrollOverlayState	(FALSE,	CSize(720, 240),		CPoint(0, 240),		TRUE,	FALSE,	TRUE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0xd0,	ALPHA_EFFECT_NONE), 1, 2);
		pHScrollOvly =		new CScrollOverlayState	(FALSE,	CSize(720, 240),		CPoint(0, 310),		FALSE,	FALSE,	TRUE,	FALSE,	0,			TRUE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0,		ALPHA_EFFECT_NONE), 1, 1);
		pTitleOvly =		new COverlayState		(FALSE,	CSize(720, 360),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			TRUE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0x80,	ALPHA_EFFECT_RTOL));
		break;

	case 4:		

		strcpy(m_szPresetDesc, "Config 5");

		bShowBackGnd = FALSE;
		bBackGndFlickerFilter = FALSE;
		bBkGndUpdateAlpha = FALSE;

		AlphaOptions.AlphaValue = 0;
		AlphaOptions.bHorzGradient = FALSE;
		AlphaOptions.bVertGradient = FALSE;
		AlphaOptions.bInvertAlpha = FALSE;
		AlphaOptions.bTransitionAlpha = FALSE;
		AlphaOptions.TransitionVerticals = 30;

		DeletePreset();
		pYUVPriVideoOvly =	new CVideoOverlayState	(FALSE,	CSize(720, 480),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pYUVSecVideoOvly =	new CVideoOverlayState	(FALSE,	CSize(720, 480),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pRGBVideoOvly =		new CVideoOverlayState	(FALSE, CSize(720/2, 480/2),	CPoint(360, 0),		FALSE,	FALSE,	FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pVScrollOvly =		new CScrollOverlayState	(FALSE, CSize(720, 160),		CPoint(0, 250),		FALSE,	FALSE,	TRUE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), 1, 2);
		pHScrollOvly =		new CScrollOverlayState	(FALSE, CSize(720, 60),			CPoint(0, 410),		FALSE,	FALSE,	TRUE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), 2, 1);
		pTitleOvly =		new COverlayState		(FALSE, CSize(313, 53),			CPoint(200, 240),	FALSE,	FALSE,	FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE));
		break;
	case 5:		

		strcpy(m_szPresetDesc, "Config 6");

		bShowBackGnd = FALSE;
		bBackGndFlickerFilter = FALSE;
		bBkGndUpdateAlpha = FALSE;

		AlphaOptions.AlphaValue = 0;
		AlphaOptions.bHorzGradient = FALSE;
		AlphaOptions.bVertGradient = FALSE;
		AlphaOptions.bInvertAlpha = FALSE;
		AlphaOptions.bTransitionAlpha = FALSE;
		AlphaOptions.TransitionVerticals = 30;

		DeletePreset();
		pYUVPriVideoOvly =	new CVideoOverlayState	(FALSE,	CSize(720, 480),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pYUVSecVideoOvly =	new CVideoOverlayState	(FALSE,	CSize(720, 480),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pRGBVideoOvly =		new CVideoOverlayState	(FALSE, CSize(720/2, 480/2),	CPoint(360, 0),		FALSE,	FALSE,	FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pVScrollOvly =		new CScrollOverlayState	(FALSE, CSize(720, 160),		CPoint(0, 250),		FALSE,	FALSE,	TRUE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), 1, 2);
		pHScrollOvly =		new CScrollOverlayState	(FALSE, CSize(720, 60),			CPoint(0, 410),		FALSE,	FALSE,	TRUE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE), 2, 1);
		pTitleOvly =		new COverlayState		(FALSE, CSize(313, 53),			CPoint(200, 240),	FALSE,	FALSE,	FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "", 0, ALPHA_EFFECT_NONE));
		break;

	case 6:		//PAL

		strcpy(m_szPresetDesc, "PAL");

		bShowBackGnd = FALSE;
		bBackGndFlickerFilter = FALSE;
		bBkGndUpdateAlpha = FALSE;

		AlphaOptions.AlphaValue = 0xc0;
		AlphaOptions.bHorzGradient = FALSE;
		AlphaOptions.bVertGradient = FALSE;
		AlphaOptions.bInvertAlpha = FALSE;
		AlphaOptions.bTransitionAlpha = FALSE;
		AlphaOptions.TransitionVerticals = 30;

		DeletePreset();
		pYUVPriVideoOvly =	new CVideoOverlayState	(TRUE,	CSize(720, 310),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0,		ALPHA_EFFECT_NONE), CSize(720, 576), CPoint(0, 0));
		pYUVSecVideoOvly =	new CVideoOverlayState	(FALSE, CSize(720, 576),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0,		ALPHA_EFFECT_NONE), CSize(720, 576), CPoint(0, 0));
		pRGBVideoOvly =		new CVideoOverlayState	(FALSE, CSize(720/2, 576/2),	CPoint(360, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",					0,		ALPHA_EFFECT_NONE), CSize(720, 576), CPoint(0, 0));
		pVScrollOvly =		new CScrollOverlayState	(TRUE,	CSize(720, 288),		CPoint(0, 288),		TRUE,	FALSE,	TRUE,	FALSE,	0,			FALSE,	CGraphicsContent(SRC_BMP_FILE, ALPHA_SRC_GENERATE, "prevue.bmp",		0xd0,	ALPHA_EFFECT_NONE), 1, 2);
		pHScrollOvly =		new CScrollOverlayState	(TRUE,	CSize(720, 288),		CPoint(0, 288),		FALSE,	FALSE,	TRUE,	FALSE,	0,			TRUE,	CGraphicsContent(SRC_BMP_FILE, ALPHA_SRC_GENERATE, "tvguidlogo.bmp",	0,		ALPHA_EFFECT_NONE), 1, 1);
		pTitleOvly =		new COverlayState		(TRUE,	CSize(720, 360),		CPoint(0, 0),		FALSE,	FALSE,	FALSE,	FALSE,	0,			TRUE,	CGraphicsContent(SRC_BMP_FILE, ALPHA_SRC_GENERATE, "tvguidlogo3.bmp",	0x80,	ALPHA_EFFECT_RTOL));
		break;


	case 7:		

		strcpy(m_szPresetDesc, "Config 8");

		bShowBackGnd = FALSE;
		bBackGndFlickerFilter = FALSE;
		bBkGndUpdateAlpha = FALSE;

		AlphaOptions.AlphaValue = 0xff;
		AlphaOptions.bHorzGradient = FALSE;
		AlphaOptions.bVertGradient = FALSE;
		AlphaOptions.bInvertAlpha = TRUE;
		AlphaOptions.bTransitionAlpha = FALSE;
		AlphaOptions.TransitionVerticals = 30;

		DeletePreset();
		pYUVPriVideoOvly =	new CVideoOverlayState	(FALSE,	CSize(720, 480),		CPoint(0, 0),		FALSE,	FALSE, FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",			0,		ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pYUVSecVideoOvly =	new CVideoOverlayState	(FALSE, CSize(720, 240),		CPoint(0, 0),		FALSE,	FALSE, FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",			0,		ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pRGBVideoOvly =		new CVideoOverlayState	(FALSE,	CSize(720, 480),		CPoint(0, 0),		FALSE,	FALSE, FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_GENERATE, ALPHA_SRC_GENERATE, "",			0,		ALPHA_EFFECT_NONE), CSize(720, 480), CPoint(0, 0));
		pVScrollOvly =		new CScrollOverlayState	(FALSE,	CSize(720, 240),		CPoint(0, 240),		TRUE,	FALSE, TRUE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_BMP_FILE, ALPHA_SRC_GENERATE, "prevue.bmp", 128,	ALPHA_EFFECT_NONE), 1, 2);
		pHScrollOvly =		new CScrollOverlayState	(FALSE,	CSize(720, 160),		CPoint(0, 320),		FALSE,	FALSE, TRUE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_BMP_FILE, ALPHA_SRC_GENERATE, "prevue.bmp", 128,	ALPHA_EFFECT_NONE), 1, 2);
		pTitleOvly =		new COverlayState		(FALSE, CSize(313, 53),			CPoint(200, 240),	FALSE,	FALSE, FALSE,	FALSE, 0,			FALSE, CGraphicsContent(SRC_BMP_FILE, ALPHA_SRC_GENERATE, "title.bmp",	128,	ALPHA_EFFECT_NONE));
		break;
	}
}


void CCardDlg::SavePreset(int PresetNum)
{
	char szPresetBaseName[150];
	char szPresetItemName[150];

	CWinApp* pApp = AfxGetApp();

	sprintf(szPresetBaseName, "Settings\\Card%d\\Preset%d", m_CardNumber, PresetNum);

	pApp->WriteProfileString(szPresetBaseName, "Description", m_szPresetDesc);

		// Save background (primary surface) settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\PrimarySurface");

	pApp->WriteProfileInt(szPresetItemName, "ShowBackGnd", (int)bShowBackGnd);
	pApp->WriteProfileInt(szPresetItemName, "FilterBackGnd", (int)bBackGndFlickerFilter);
	pApp->WriteProfileInt(szPresetItemName, "WriteAlphaToBackGnd", (int)bBkGndUpdateAlpha);
	pApp->WriteProfileInt(szPresetItemName, "AlphaValue", (int)AlphaOptions.AlphaValue);
	pApp->WriteProfileInt(szPresetItemName, "UseHorzAlphaGradient", (int)AlphaOptions.bHorzGradient);
	pApp->WriteProfileInt(szPresetItemName, "UseVertAlphaGradient", (int)AlphaOptions.bVertGradient);
	pApp->WriteProfileInt(szPresetItemName, "InvertAlpha", (int)AlphaOptions.bInvertAlpha);
	pApp->WriteProfileInt(szPresetItemName, "TransitionAlpha", (int)AlphaOptions.bTransitionAlpha);
	pApp->WriteProfileInt(szPresetItemName, "AlphaTransitionVerticals", (int)AlphaOptions.TransitionVerticals);

	PriSurfaceContents.SaveToRegistry(szPresetItemName);

		// Save Primary video window settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\YUVPrimaryVideoWindow");

	if (pYUVPriVideoOvly) {
		pYUVPriVideoOvly->SaveToRegistry(szPresetItemName);
	}

		// Save Secondary video window settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\YUVSecondaryVideoWindow");

	if (pYUVSecVideoOvly) {
		pYUVSecVideoOvly->SaveToRegistry(szPresetItemName);
	}

		// Save RGB video window settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\RGBVideoWindow");

	if (pRGBVideoOvly) {
		pRGBVideoOvly->SaveToRegistry(szPresetItemName);
	}

		// Save Vertical Scroll window settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\VerticalScrollWindow");

	if (pVScrollOvly) {
		pVScrollOvly->SaveToRegistry(szPresetItemName);
	}

		// Save Horizontal Scroll window settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\HorizontalScrollWindow");

	if (pHScrollOvly) {
		pHScrollOvly->SaveToRegistry(szPresetItemName);
	}

		// Save Title window settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\TitleWindow");

	if (pTitleOvly) {
		pTitleOvly->SaveToRegistry(szPresetItemName);
	}

}


void CCardDlg::UsePreset(int PresetNum)
{
	char szPresetBaseName[150];
	char szPresetItemName[150];

	DeletePreset();

	m_ConfigPreset = PresetNum;

	CWinApp* pApp = AfxGetApp();

	sprintf(szPresetBaseName, "Settings\\Card%d\\Preset%d", m_CardNumber, PresetNum);

	strcpy(m_szPresetDesc, pApp->GetProfileString(szPresetBaseName, "Description", ""));
	if (strlen(m_szPresetDesc) == 0) {
		ReadDefaultPreset(PresetNum);
		return;
	}

		// Read background (primary surface) settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\PrimarySurface");

	bShowBackGnd = (BOOL)pApp->GetProfileInt(szPresetItemName, "ShowBackGnd", FALSE);
	bBackGndFlickerFilter = (BOOL)pApp->GetProfileInt(szPresetItemName, "FilterBackGnd", FALSE);
	bBkGndUpdateAlpha = (BOOL)pApp->GetProfileInt(szPresetItemName, "WriteAlphaToBackGnd", FALSE);
	AlphaOptions.AlphaValue = pApp->GetProfileInt(szPresetItemName, "AlphaValue", 0);
	AlphaOptions.bHorzGradient = (BOOL)pApp->GetProfileInt(szPresetItemName, "UseHorzAlphaGradient", FALSE);
	AlphaOptions.bVertGradient = (BOOL)pApp->GetProfileInt(szPresetItemName, "UseVertAlphaGradient", FALSE);
	AlphaOptions.bInvertAlpha = (BOOL)pApp->GetProfileInt(szPresetItemName, "InvertAlpha", FALSE);
	AlphaOptions.bTransitionAlpha = (BOOL)pApp->GetProfileInt(szPresetItemName, "TransitionAlpha", FALSE);
	AlphaOptions.TransitionVerticals = pApp->GetProfileInt(szPresetItemName, "AlphaTransitionVerticals", 0);

	PriSurfaceContents = CGraphicsContent(szPresetItemName);

		// Read Primary video window settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\YUVPrimaryVideoWindow");

	pYUVPriVideoOvly = new CVideoOverlayState(szPresetItemName);

		// Read Secondary video window settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\YUVSecondaryVideoWindow");

	pYUVSecVideoOvly = new CVideoOverlayState(szPresetItemName);

		// Read RGB video window settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\RGBVideoWindow");

	pRGBVideoOvly =	new CVideoOverlayState(szPresetItemName);

		// Read Vertical Scroll window settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\VerticalScrollWindow");

	pVScrollOvly = new CScrollOverlayState(szPresetItemName);

		// Read Horizontal Scroll window settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\HorizontalScrollWindow");

	pHScrollOvly = new CScrollOverlayState(szPresetItemName);

		// Read Title window settings
	strcpy(szPresetItemName, szPresetBaseName);
	strcat(szPresetItemName, "\\TitleWindow");

	pTitleOvly = new COverlayState(szPresetItemName);
}







