// CardDlg.h : header file
//

#ifndef _CARD_DLG_H_
#define _CARD_DLG_H_

#include <afxctl.h>
#include "ddraw.h"
#include "dvp.h"
#include "MPEGDecControlDlg.h"
#include "dibapi.h"

class CVerticalStatus;
class CCardDlg;

#define INIT_DIRECTDRAW_STRUCT(x) (ZeroMemory(&x, sizeof(x)), x.dwSize=sizeof(x))


/////////////////////////////////////////////////////////////////////////////
// CCardDlg dialog

#define PREVUE_DEMO_SETTINGS	0			// Configuration preset for Prevue demo

#define MAX_VIDEO_PORTS	2
#define PRIMARY_VIDEO_PORT		1
#define SECONDARY_VIDEO_PORT	0

#define NUM_VID_SWITCH_BMPS		8

#define LEN_PRESET_DESC			30

typedef enum {								// Alpha handling options for bitmap copy routine
	ALPHA_EFFECT_NONE,
	ALPHA_EFFECT_LTOR,
	ALPHA_EFFECT_RTOL,
	ALPHA_EFFECT_TTOB,
	ALPHA_EFFECT_BTOT
} ALPHA_EFFECT;


typedef struct {
	int AlphaValue;
	BOOL bHorzGradient;
	BOOL bVertGradient;
	BOOL bInvertAlpha;
	BOOL bTransitionAlpha;
	int TransitionVerticals;
} ALPHA_OPTIONS, *PALPHA_OPTIONS;


typedef enum {
	SRC_GENERATE = 0,
	SRC_BMP_FILE
} GRAPHICS_SOURCE;


typedef enum {
	ALPHA_SRC_GENERATE = 0,
	ALPHA_SRC_BMP_FILE
} ALPHA_SOURCE;

typedef struct {
	HDIB hDib;
	BYTE *pDIBBits; 
	LPBITMAPINFO pDIBHdr;
	DWORD BytesPerPixel;
	DWORD BytesPerRow;
	DWORD dwWidth;
	DWORD dwHeight;
} SCROLL_DIB_INFO;

// Classes for keeping up with test overlay states

class CGraphicsContent
{
public:
	CGraphicsContent();
	CGraphicsContent(char* pszBaseKeyName);
	CGraphicsContent(GRAPHICS_SOURCE Source, ALPHA_SOURCE AlphaSource, char* Filename,
		int StartingAlpha, ALPHA_EFFECT AlphaEffect);
	GRAPHICS_SOURCE m_Source;
	ALPHA_SOURCE m_AlphaSource;
	char m_Filename[_MAX_PATH];		
	int m_StartingAlpha;
	ALPHA_EFFECT m_AlphaEffect;	

	SaveToRegistry(char* pszBaseKeyName);
};

class COverlayState 
{
public:
	COverlayState();
	COverlayState(char* pszBaseKeyName);
	COverlayState(BOOL bShow, CSize Size, CPoint Position, BOOL bBlend, BOOL bInvBlend, BOOL bFilter, BOOL bCkEnable, DWORD CkColor, BOOL bWriteAlpha, CGraphicsContent Content);
	BOOL m_bShow;
	CSize m_Size;
	CPoint m_Position;
	BOOL m_bBlend;
	BOOL m_bInvBlend;
	BOOL m_bFilter;
	BOOL m_bCkEnable;
	DWORD m_CkColor;
	BOOL m_bWriteAlpha;
	CGraphicsContent m_Content;

	SaveToRegistry(char* pszBaseKeyName);
};


class CVideoOverlayState: public COverlayState
{
public:
	CVideoOverlayState();
	CVideoOverlayState(char* pszBaseKeyName);
	CVideoOverlayState(BOOL bShow, CSize Size, CPoint Position, BOOL bBlend, BOOL bInvBlend, BOOL bFilter, BOOL bCkEnable, DWORD CkColor, BOOL bWriteAlpha, CGraphicsContent Content,
		CSize SrcCropSize, CPoint SrcCropPosition);
	CSize m_SrcCropSize;
	CPoint m_SrcCropPosition;

	SaveToRegistry(char* pszBaseKeyName);
};


class CScrollOverlayState: public COverlayState
{
public:
	CScrollOverlayState();
	CScrollOverlayState(char* pszBaseKeyName);
	CScrollOverlayState(BOOL bShow, CSize Size, CPoint Position, BOOL bBlend, BOOL bInvBlend, BOOL bFilter, BOOL bCkEnable, DWORD CkColor, BOOL bWriteAlpha, CGraphicsContent Content,
		int PixelsPerStep, int VerticalsPerStep);
	int m_PixelsPerStep;
	int m_VerticalsPerStep;

	SaveToRegistry(char* pszBaseKeyName);

};



class CCardDlg : public CPropertyPage
{
	DECLARE_DYNCREATE(CCardDlg)
	
// Construction
public:
	CCardDlg(int CardNumber = 0);
	~CCardDlg();

	DWORD DbgVScrollPos;

	friend class CMpegStatus;
	friend class CVerticalStatus;
	CMpegStatus *pMpegStatus;
	CVerticalStatus *pVerticalStatus;

	void SaveSettings();
	void SavePreset(int PresetNum);
	void UsePreset(int PresetNum);
	void ShowOverlay(int Channel);
	void ReadDefaultPreset(int PresetNum);

	void ResetCard();
	void SetupRGBProc();
	void SetupYUVProc();
	void ApplyGraphicsSettings();
	void DisableGraphicsSettings();

	void FillSurface(LPDIRECTDRAWSURFACE lpDDSurface, DWORD color);
	void TestSurfaceAlloc(void);

	void GetAlphaSettings(ALPHA_OPTIONS* pAlphaOptions);
	void ApplyAlphaSettings(BOOL bUseBackBuffer);


	int m_CardNumber;
	int m_ConfigPreset;					// Current configuration setting

	VIDEO_MODE m_VideoMode;				// Current video mode
	VIDEO_MODE m_LastVideoMode;			// Last video mode used

	TULSA_CONFIG m_Config;				// Current board configuration

	DWORD m_hStream[NUM_SUB_COMMANDS];	// Stream handle

	BOOL m_bBypass;						// TRUE if board outputs are bypassed
	BOOL m_bACCEnable;					// TRUE if decoder ACC mode enabled


	UINT m_TimerID;						// Timer used for period status updates

	BOOL m_bVSUpdate;					// TRUE if vertical scrolling should be updated using new settings
	BOOL m_bHSUpdate;					// TRUE if horz scrolling should be updated using new settings

	SCROLL_DIB_INFO m_VScrollDibInfo;	// Info for DIB used for VScroll
	SCROLL_DIB_INFO m_HScrollDibInfo;	// Info for DIB used for HScroll
//	SCROLL_DIB_INFO m_VScrollTimeBarDibInfo;

	char m_szPresetDesc[LEN_PRESET_DESC];// Description settings

	CVideoOverlayState *pYUVPriVideoOvly;
	CVideoOverlayState *pYUVSecVideoOvly;
	CVideoOverlayState *pRGBVideoOvly;
	CScrollOverlayState *pVScrollOvly;
	CScrollOverlayState *pHScrollOvly;
	COverlayState *pTitleOvly;

	DWORD VScrollZOrderFlags;
	DWORD HScrollZOrderFlags;
	DWORD RGBVideoZOrderFlags;
	DWORD TitleOvlyZOrderFlags;

	int HorzDisplayCount;
	BOOL bEnableHorzScrollThread;
	BOOL bHorzScrollThreadFinished;
	HANDLE hEventHorzScroll;
	HANDLE hEventHorzFF;
	BOOL HScrollReady;

	BOOL bEnableVertScrollThread;
	BOOL bVertScrollThreadFinished;
	HANDLE hEventVertScroll;
	HANDLE hEventVertFF;
	BOOL VScrollReady;
	int VertDisplayCount; 

	DWORD BackGroundColor;				// Color used for screen background

	BOOL bShowBackGnd;					// TRUE if background bitmap should be shown
	BOOL bBackGndFlickerFilter;			// TRUE if background bitmap should be flicker filtered
	BOOL bBkGndUpdateAlpha;				// TRUE if alpha channel should be updated when painting bitmap

	CGraphicsContent PriSurfaceContents;// Graphics used for primary surface


	ALPHA_OPTIONS AlphaOptions;			// Alpha channel options
	int m_bLastAlpha;

	CSize TitleOverlaySize;				// Size of title overlay

	BOOL bVideoTestStartedYUV[MAX_VIDEO_PORTS]; // TRUE if video test started on YUV proc 
	BOOL bVideoTestStartedRGB;			// TRUE if video test started on RGB proc

	ISTBTAudioControl				*pAudioControl;
	ISTBTAnalogVideoDecoderControl	*pAnalogVideoDecoderControl;
	ISTBTAnalogVideoEncoderControl	*pAnalogVideoEncoderControl;
	ISTBTBoardControl				*pBoardControl;
	ISTBTGenlockControl				*pGenlockControl;
	ISTBTMPEGDecoderControl			*pMPEGDecoderControl;
	ISTBTVBIControl					*pVBIControl;
	ISTBTVideoControl				*pVideoControl;
	ISTBTTest						*pTest;
	ISTBTDebug						*pDebug;

	LPDIRECTDRAW lpDirectDrawRGB;
	LPDIRECTDRAW lpDirectDrawYUV;

	LPDIRECTDRAW2 lpDD2RGB;
	LPDIRECTDRAW2 lpDD2YUV;

	LPDIRECTDRAWSURFACE lpDDSPrimaryRGB;
	LPDIRECTDRAWSURFACE lpDDSPrimaryYUV;

	LPDIRECTDRAWVIDEOPORT lpDDVideoPortYUV[MAX_VIDEO_PORTS];
	LPDIRECTDRAWVIDEOPORT lpDDVideoPortRGB;

	LPDIRECTDRAWSURFACE lpDDSOverlayVideo;
	LPDIRECTDRAWSURFACE lpDDSOverlayVScroll;
	LPDIRECTDRAWSURFACE lpDDSOverlayHScroll;
	LPDIRECTDRAWSURFACE lpDDSTitle;
	LPDIRECTDRAWSURFACE lpDDSTest;
	LPDIRECTDRAWSURFACE lpDDSOverlayTimebar;

// Dialog Data
	//{{AFX_DATA(CCardDlg)
	enum { IDD = IDD_CARD };
	//}}AFX_DATA


// Overrides
	// ClassWizard generate virtual function overrides
	//{{AFX_VIRTUAL(CCardDlg)
	public:
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

private:

	CPictureHolder m_PictureAudSwitch[2];
	CPictureHolder m_PictureVidSwitch[NUM_VID_SWITCH_BMPS];
	CPictureHolder m_PictureAudBypass[2];
	CPictureHolder m_PictureVidBypass[2];

	void SetAnalogAudioVolume();
	void GetAnalogAudioVolume();
	void SetMPEGAudioVolume();
	void GetMPEGAudioVolume();
	void SetAudioSwitch();
	void GetAudioSwitch();
	void SetVideoSwitch();
	void GetVideoSwitch();
	void SetAudioDelay();
	void GetAudioDelay();
	void SetBypassSwitch();
	void GetBypassSwitch();
	void SetACCMode();
	void GetACCMode();

	char* PrependCardID(char* buffer, char* string);
	TULSA_RESULT InitCard(void);

	void GetCntlRect(int ID, CRect &rect);

	void StartVideoTestYUV(int Port);
	void StopVideoTestYUV(int Port);
	void StopVideoTestAllYUV(void);
	void UpdateVideoTestYUV(int Port);

	void StartVideoTestRGB(void);
	void StopVideoTestRGB(void);
	void UpdateVideoTestRGB(void);

	void ShowTitleOverlay(void);
	void DeleteTitleOverlay(void);
	void UpdateTitleOverlay(void);

	void CopyBitmap(DDSURFACEDESC* pSurface, int BitmapId, DWORD AlphaValue, ALPHA_EFFECT AlphaEffect, BOOL bUpdateAlphaChannel);
	void CopyBitmapFromFile(DDSURFACEDESC* pSurface, CGraphicsContent *pGraphicsContent, BOOL bUpdateAlphaChannel);

	void DrawYUVColorBars(LPDIRECTDRAWSURFACE lpDDSurface);
	void DrawTestPattern(DDSURFACEDESC* pSurface, COLORREF color);

	void StopPlayback();

	void DeletePreset();

	void StopVertScrollTest(void);
	void StartVertScrollTest(void);
	void UpdateVertScrollTest(void);
	BOOL VertScrollEnabled(void);
friend UINT VertScrollThread(LPVOID lpvParam);

	void StopHorzScrollTest(void);
	void StartHorzScrollTest(void);
	void UpdateHorzScrollTest(void);
	BOOL HorzScrollEnabled(void);
friend UINT HorzScrollThread(LPVOID lpvParam);

	void ReadScrollBitmap(char* szFilename, SCROLL_DIB_INFO &ScrollDIBInfo);
	void FreeScrollBitmap(SCROLL_DIB_INFO &ScrollDIBInfo);

// Implementation
protected:
	LRESULT UpdateStatus(WPARAM wParam, LPARAM lParam);

	// Generated message map functions
	//{{AFX_MSG(CCardDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnReset();
	afx_msg void OnConfig();
	afx_msg void OnDestroy();
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnChangeAnalogAudVol();
	afx_msg void OnChangeAudDelay();
	afx_msg void OnUseMpegAud();
	afx_msg void OnUseAnalogAud();
	afx_msg void OnChangeMpegAudVol();
	afx_msg void OnSelendokPriVidPortSrcYUV();
	afx_msg void OnSelendokSecVidPortSrcYUV();
	afx_msg void OnBypass();
	afx_msg void OnPaint();
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnGraphicsOptions();
	afx_msg void OnVideoEncoderOptions();
	afx_msg void OnAccOn();
	afx_msg void OnMpegDecControl();
	afx_msg void OnSelendokVidPortSrcRGB();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


class CVerticalStatus : ISTBTVerticalStatus
{
public:
	CVerticalStatus(CCardDlg *pParent) : m_pParent(pParent) {}
	CCardDlg *m_pParent;

     //*** IUnknown methods ***   
    CRefCount				m_cRef;   // object reference counter
    STDMETHODIMP            QueryInterface(REFIID riid, LPVOID FAR *ppv) 
										{ QUERY_MY_INTERFACE(IID_ISTBTVerticalStatus, ISTBTVerticalStatus) }
    STDMETHODIMP_(ULONG)    AddRef()	{ return(m_cRef.AddRef()); }
    STDMETHODIMP_(ULONG)    Release()	{ return(m_cRef.Release()); }
    /*** ISTBTMPEGStatus methods ***/
    STDMETHODIMP			VerticalOccurred(
								DWORD VerticalNumber);
};

#endif
